@@ -1,4 +1,4 @@
-# Copyright (C) 2017 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2017-2019 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -22,18 +22,15 @@ to download them.
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2017, Damon Lynch"
+__copyright__ = "Copyright 2017-2019, Damon Lynch"
 
 import logging
-import pkg_resources
 from collections import namedtuple
-import shlex
+import pkg_resources
 import hashlib
 import os
 import traceback
 import shutil
-import sys
-import subprocess
 from typing import Optional
 
 from gettext import gettext as _
@@ -50,7 +47,7 @@ from raphodo.constants import (
     remote_versions_file, CheckNewVersionDialogState, CheckNewVersionDialogResult,
     standardProgressBarWidth
 )
-from raphodo.utilities import (create_temp_dir, format_size_for_user)
+from raphodo.utilities import (create_temp_dir, format_size_for_user, is_venv)
 from raphodo.interprocess import ThreadNames
 from raphodo.viewutils import translateButtons
 
@@ -63,7 +60,7 @@ class NewVersion(QObject):
     Runs in its own thread.
     """
 
-    checkMade = pyqtSignal(bool, version_details, version_details, str, bool, bool)
+    checkMade = pyqtSignal(bool, version_details, version_details, str, bool, bool, bool)
     # See http://pyqt.sourceforge.net/Docs/PyQt5/signals_slots.html#the-pyqt-pyobject-
     # signal-argument-type
     bytesDownloaded = pyqtSignal('PyQt_PyObject')  # don't convert python int to C++ int
@@ -81,7 +78,9 @@ class NewVersion(QObject):
         self.rapidApp.checkForNewVersionRequest.connect(self.check)
         self.rapidApp.downloadNewVersionRequest.connect(self.download)
         self.installed_via_pip_check_made = False
-        self.installed_via_pip = None  # type: bool
+        self.is_venv_check_made = False
+        self.installed_via_pip = None  # type: Optional[bool]
+        self.is_venv = None  # type: Optional[bool]
 
     @pyqtSlot()
     def start(self) -> None:
@@ -95,23 +94,18 @@ class NewVersion(QObject):
 
         Exceptions are not caught.
 
-        Calling pip directly as python code and redirecting stdout
-        totally messes up debugging output, even when using a context
-        manager. So don't do that!
-
         :param package: package name to search for
         :return: True if installed via pip, else False
         """
 
-        command_line = '{} -m pip show --disable-pip-version-check --verbose {}'.format(
-            sys.executable, package
-        )
-        args = shlex.split(command_line)
-        try:
-            pip_output = subprocess.check_output(args)
-        except subprocess.SubprocessError:
-            return False
-        return pip_output.decode().find('Installer: pip') >= 0
+        pip_install = False
+        pkg = pkg_resources.get_distribution(package)
+        if pkg.has_metadata('INSTALLER'):
+            if pkg.get_metadata('INSTALLER').strip() == 'pip':
+                pip_install = True
+
+        logging.debug("Installed using pip: %s", pip_install)
+        return pip_install
 
     @pyqtSlot()
     def check(self) -> None:
@@ -161,13 +155,20 @@ class NewVersion(QObject):
             try:
                 self.installed_via_pip = self.installedUsingPip()
             except Exception:
-                logging.warning(
-                    "Exception encountered when checking if pip was used to install"
+                logging.debug(
+                    "Exception encountered when checking if pip was used to install "
+                    "(probably the program has not been installed)"
                 )
                 self.installed_via_pip = False
 
-        self.checkMade.emit(success, stable_version, dev_version, download_page, no_upgrade,
-                            self.installed_via_pip)
+        if not self.is_venv_check_made:
+            self.is_venv = is_venv()
+            self.is_venv_check_made = True
+
+        self.checkMade.emit(
+            success, stable_version, dev_version, download_page, no_upgrade, self.installed_via_pip,
+            self.is_venv
+        )
 
     def verifyDownload(self, downloaded_tar: str, md5_url: str) -> bool:
         """
